const randomstring = require('randomstring');
const {User} = require('@codefresh-io/authenticated-entity');
const { mongoClient } = require('@codefresh-io/service-base');

const mongoUri = process.env.MONGO_URI || 'mongodb://codefresh.dev:27017';

function initTestUtils(dropTestDatabase = true) {
  const dbName = `unit_tests_${randomstring.generate(5)}`;
  let mock_user = null;

  jest.mock('@codefresh-io/service-base', () => {
    const serviceBase = require.requireActual('@codefresh-io/service-base');
    serviceBase.getAuthenticatedEntity = () => mock_user;
    return serviceBase;
  });

  beforeAll(() => {
    return mongoClient.init({mongo: {uri: `${mongoUri}/${dbName}`}})
  });

  afterEach(() => {
    if (dropTestDatabase) {
      return mongoClient.db.dropDatabase();
    }
  });

  return {
    setAuthenticatedUser: (data) => { mock_user = new User(data) }
  }
}

const generateObjectId = () => new mongoClient.ObjectId().toString()

function errorTest(description, testFunc, ...words) {
  it(description, function() {
    const result = testFunc();
    return Promise.all(words.map(word => expect(result).rejects.toHaveProperty('message', expect.stringContaining(word))));
  });
}

module.exports = {
  initTestUtils,
  generateObjectId,
  errorTest
};
